"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const THREE = require("three");
const three_1 = require("three");
const GeUtils_1 = require("./GeUtils");
const Material_1 = require("./Material");
const RotateUV_1 = require("./RotateUV");
const ThreeCSG_1 = require("./ThreeCSG");
//解析二维圆弧类.
class Arc2d {
    constructor(p1, p2, bul) {
        this.m_StartPoint = p1.clone();
        this.m_EndPoint = p2.clone();
        let vec = p2.clone().sub(p1);
        let len = vec.length();
        let an = vec.angle();
        this.m_Radius = len / Math.sin(2 * Math.atan(bul)) / 2;
        let allAngle = Math.atan(bul) * 4;
        let delDis = bul * len / 2;
        let toDis = this.m_Radius - delDis;
        an += Math.PI * 0.5;
        this.m_Center = p1.clone().add(p2);
        this.m_Center.multiplyScalar(0.5);
        GeUtils_1.polar(this.m_Center, an, toDis);
        this.m_StartAn = p1.clone().sub(this.m_Center).angle();
        this.m_EndAn = p2.clone().sub(this.m_Center).angle();
        if (bul < 0) {
            //一个神奇的特性 它需要这么做
            this.m_StartAn -= Math.PI;
            this.m_EndAn -= Math.PI;
        }
    }
}
exports.Arc2d = Arc2d;
//创建轮廓 通过点表和凸度    
function createPath(pts, buls, shapeOut) {
    let shape = shapeOut || new THREE.Shape();
    if (pts.length === 0)
        return shape;
    let firstPt = pts[0];
    shape.moveTo(firstPt.x, firstPt.y);
    for (let i = 0; i < pts.length - 1; i++) {
        let nextPt = pts[i + 1];
        if (buls[i] == 0) {
            shape.lineTo(nextPt.x, nextPt.y);
        }
        else {
            let pt = pts[i];
            //参考
            //http://www.dorodnic.com/blog/tag/three-js/ 绘制一个齿轮
            //https://www.kirupa.com/html5/drawing_circles_canvas.htm //html5
            let arc2 = new Arc2d(pt, nextPt, buls[i]);
            let cen = arc2.m_Center;
            shape.absarc(cen.x, cen.y, arc2.m_Radius, arc2.m_StartAn, arc2.m_EndAn, buls[i] < 0);
        }
    }
    return shape;
}
exports.createPath = createPath;
function getVec(data) {
    return new THREE.Vector3(data["x"], data["y"], data["z"]);
}
exports.getVec = getVec;
//创建板件 暂时这么写
function createBoard(boardData) {
    let pts = new Array();
    let buls = new Array();
    let boardPts = boardData["Pts"];
    let boardBuls = boardData["Buls"];
    let boardHeight = boardData["H"];
    let boardMat = new THREE.Matrix4();
    let matInv = new THREE.Matrix4();
    //InitBoardMat
    {
        let xD = getVec(boardData["XVec"]);
        let yD = getVec(boardData["YVec"]);
        let ZD = getVec(boardData["ZVec"]);
        let pBase = getVec(boardData["BasePoint"]);
        boardMat.makeBasis(xD, yD, ZD);
        boardMat.setPosition(pBase);
        matInv.getInverse(boardMat, true);
    }
    if (boardPts && boardPts.length !== 0)
        for (let i = 0; i < boardPts.length; i++) {
            let pt = getVec(boardPts[i]);
            if (boardPts[i].z !== undefined)
                pt.applyMatrix4(matInv);
            pts.push(new THREE.Vector2(pt.x, pt.y));
            buls.push(boardBuls[i]);
        }
    else {
        let length = boardData["L"];
        let width = boardData["W"];
        let height = boardData["H"];
        pts.push(new three_1.Vector2(0, 0), new three_1.Vector2(width, 0), new three_1.Vector2(width, length), new three_1.Vector2(0, length), new three_1.Vector2(0, 0));
        buls.push(0, 0, 0, 0, 0);
    }
    let sp = createPath(pts, buls);
    let extrudeSettings = {
        steps: 1,
        bevelEnabled: false,
        amount: boardHeight
    };
    let ext = new THREE.ExtrudeGeometry(sp, extrudeSettings);
    ext.computeBoundingSphere();
    ext.computeBoundingBox();
    ext.translate(0, 0, -boardHeight);
    ext.applyMatrix(boardMat);
    //外边.
    let edges = [createEdge(ext)];
    //差集
    if (boardData["SubBoardLocal"].length > 0) {
        let subBoardList = boardData["SubBoardLocal"].map(d => createBoard(d));
        let thisCsg = new ThreeCSG_1.ThreeBSP(ext);
        for (let br of subBoardList) {
            edges.push(...br.edges);
            let subCsg = new ThreeCSG_1.ThreeBSP(br.mesh);
            thisCsg = thisCsg.subtract(subCsg);
        }
        ext = thisCsg.toGeometry();
    }
    if (boardData["BoardName"] === "地脚线")
        RotateUV_1.RotateUVs(ext);
    let mesh = new THREE.Mesh(ext, Material_1.boardMaterial);
    return { mesh, edges };
}
exports.createBoard = createBoard;
function createTemplateBoard(brDataList) {
    let meshs = [];
    let edgesa = [];
    for (let d of brDataList) {
        let { mesh, edges } = createBoard(d);
        meshs.push(mesh);
        edgesa.push(...edges);
    }
    return { meshs, edgesa };
}
exports.createTemplateBoard = createTemplateBoard;
function createEdge(geo) {
    let edge = new THREE.EdgesGeometry(geo, 1);
    return new three_1.LineSegments(edge, Material_1.edgeMaterial);
}
exports.createEdge = createEdge;
//# sourceMappingURL=createBoard.js.map